// ===========================================================================
// CONTENT  : ABSTRACT CLASS ASubWindowPlugin
// AUTHOR   : M.Duchrow
// VERSION  : 1.0 - 20/12/2006
// HISTORY  :
//  20/12/2006  mdu  CREATED
//
// Copyright (c) 2006, by M.Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.tools.cda.ui.plugin.spi ;

// ===========================================================================
// IMPORTS
// ===========================================================================
import java.awt.Frame;

import javax.swing.JFrame;

import org.pfsw.odem.IExplorationModelObject;
import org.pfsw.tools.cda.ui.xpi.AActionPlugin;
import org.pfsw.tools.cda.xpi.IIntegrationService;
import org.pfsw.tools.cda.xpi.PluginConfiguration;

/**
 * This abstract class specifies the methods to be implemented by all subclasses
 * that are intended to provide a sub-window to the main application.
 * <p>
 * Each concrete subclass must specifiy a public non-argument constructor.
 *
 * @author M.Duchrow
 * @version 1.0
 */
abstract public class ASubWindowPlugin extends AActionPlugin
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================

  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================
  private IIntegrationService integrationService = null ;
  /**
   * Returns the integration service that is the interface to the main application.
   * Its purpose is to provide all necessary data from the main application and
   * functionality which might be useful in plug-ins as well. 
   */
  public IIntegrationService getIntegrationService() { return integrationService ; }
  /**
   * Do NOT call this method. It is public because it is used to initialize 
   * the plug-in.  
   */
  public void setIntegrationService( IIntegrationService newValue ) { integrationService = newValue ; }

  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
  /**
   * Initialize the new instance with default values.
   */
  protected ASubWindowPlugin()
  {
    super();
  } // ASubWindowPlugin() 

  // -------------------------------------------------------------------------

  // =========================================================================
  // ABSTRACT PUBLIC INSTANCE METHODS
  // =========================================================================
  /**
   * This method must be implemented to create the whole UI of the new window.
   * It returns a Frame containing all widgets that are necessary to show whatever
   * the plug-in wants to show.
   * 
   * @param object The element on which this plug-in was invoked
   * @return A frame or null. If null is returned the plug-in execution get cancelled.
   */
  abstract public JFrame createFrame(IExplorationModelObject object);

  // -------------------------------------------------------------------------

  // =========================================================================
  // PUBLIC INSTANCE METHODS
  // =========================================================================
  /**
   * Returns the configuration data that might be necessary to execute
   * the plugin's functionality
   * 
   * @param parent The parent frame from which the plugin is used
   */
  @Override
  public PluginConfiguration getConfiguration(Frame parent)
  {
    return new PluginConfiguration();
  } // getConfiguration() 

  // -------------------------------------------------------------------------

  /**
   * This method will be called right after the plug-in has been created
   * and connected to the internal structures.
   * The purpose of this method is to do all necessary initialization.
   * Do NOT use the constructor for initialization because the plug-in is not 
   * connected and basically initialized at that time.
   * <br>
   * Instead use this method.
   * <p>
   * This is an empty implementation of this method which is convenient for
   * subclasses that don't need an initialization at all.
   */
  @SuppressWarnings("unused")
  public void initialize(PluginConfiguration config)
  {
    // Nothing to init here
  } // initialize() 

  // -------------------------------------------------------------------------

  /**
   * This method will be called right before the plug-in gets closed.
   * Its purpose is to release all resources and clean-up everything to avoid
   * memory leaks.
   * <br>
   * Subclasses that override this method should always call super.dispose()
   * at the end of the overriden method.
   * <p>
   * This here is an empty implementation of this method which is convenient 
   * for subclasses that don't need to dispose anything..
   */
  public void dispose()
  {
    // Nothing to dispose here
  } // dispose()

  // -------------------------------------------------------------------------

  /**
   * This method must be called in order to close this plug-in with its window
   * and automatically inform its controller.
   */
  public void close()
  {
    if (this.getIntegrationService() != null)
    {
      this.getIntegrationService().executionFinished();
    }
  } // close()

  // -------------------------------------------------------------------------

  // =========================================================================
  // PROTECTED INSTANCE METHODS
  // =========================================================================

} // class ASubWindowPlugin 
